package asd.td3;

/**
 * Une classe pour l'exercice 3 de la feuille 3.
 * On teste rcursivement si on peut faire la somme i avec certaines des
 * valeurs de T, un tableau d'entiers.
 *  2002 Marc Gatano
 */
public class Somme {

    private int[] T; // le tableau d'entiers positifs
    private int N;   // la somme des lments de T
    
    /**
     * Construit un objet Somme.
     * Prcondition : table[i] >= 0 pour tout i dans [0..T.length-1].
     * Complexit : O(n) o n = T.length.
     */
    public Somme(int[] table) {
        T = table;
        N = 0;
        
        for ( int i = 0; i < T.length; i++ ) {
            N = N + T[i];
        }
    }

    /**
     * Teste si on peut faire la somme i avec un sous-ensembles d'entiers
     * appartenant  T.
     * Complexit : O(2^n) o n = T.length.
     */
    public boolean possible(int i) {
        if ( i < 0 || i > N ) {
            return false;
        }
        return possible(i,T.length);
    }

    /**
     * Teste si on peut faire la somme i avec un sous-ensemble d'entiers
     * appartenant  T[0..k-1].
     * Prconditions : 0 <= i <= N et 0 <= k < T.length.
     * Complexit : O(2^k).
     */
    private boolean possible(int i, int k) {
        if ( i == 0 ) {
            return true;
        }
        if ( k == 0 ) {
            return false;
        }
        if ( T[k-1] <= i ) {
            return possible(i,k - 1) || possible(i - T[k-1],k - 1);
        }
        return possible(i,k - 1);
    }
    // On remarque que si T[i] = 0 pour tout i, la complexit de possible(i,k)
    // ne dpend que de k. Soit C(k) cette complexit, on a alors :
    // C(k) = 2*C(k-1) = 2^k*C(0) = 2^k (car C(0) = 1).
}
