package asd.td3;

/**
 * Une classe pour l'exercice 4 de la feuille 3.
 * On calcule rcursivement la valeur maximale du butin que le voleur peut
 * emporter en choisissant des objets parmis N objets, l'objet i ayant le
 * poids P[i] et la valeur V[i], pour tout i dans [0..N-1], et sachant que
 * le voleur ne peut pas emporter un poids suprieur  Pmax.
 *  2002 Marc Gatano
 */
public class Voleur {

    private int[] P;  // P[i] = le poids de l'objet i (P[i] > 0 pour tout i).
    private int[] V;  // V[i] = la valeur de l'objet i.
    
    /**
     * Construit un objet Voleur.
     * Prconditions : P.length = V.length, et P[i] et V[i] > 0 pout tout i
     * dans [0..P.length-1].
     * Complexit : O(1).
     */
    public Voleur(int[] P, int[] V) {
        this.P = P;
        this.V = V;
    }

    /**
     * Retourne la valeur maximale du butin que le voleur peut emporter
     * sans dpasser le poids total Pmax.
     * Complexit : O(2^n) o n = T.length.
     */
    public int valeur(int Pmax) {
        return valeur(Pmax,P.length);
    }

    /**
     * Retourne la valeur maximale du butin que le voleur peut emporter
     * sans dpasser le poids total Pmax et en choisissant parmis les objets
     * de numro [0..k-1].
     * Complexit : O(2^k).
     */
    public int valeur(int Pmax, int k) {
        if ( Pmax == 0 || k == 0 ) { // ou ( k == 0 ) seulement
            return 0;
        }
        if ( P[k-1] <= Pmax ) {
            return max(valeur(Pmax,k-1),valeur(Pmax - P[k-1],k-1) + V[k-1]);
        }
        return valeur(Pmax,k-1);
    }
    // On remarque que si P[i] = 0 pour tout i, la complexit de valeur(Pmax,k)
    // ne dpend que de k. Soit C(k) cette complexit, on a alors :
    // C(k) = 2*C(k-1) = 2^k*C(0) = 2^k (car C(0) = 1).

    private int max(int i, int j) {
        if ( i > j ) {
            return i;
        }
        return j;
    }
}
